/*
 * transducer_cli_header.c
 *
 *  Created on: 16 mai 2013
 *      Author: rj092
 */

#ifdef WIN32
#include "../../Interface_Library/msxe370x_modbus_clientlib.h"
#else
#include <msxe370x_modbus_clientlib.h>
#endif

#include <addidata_console.h>
#include <addidata_network.h>
#include <addidata_dataclient.h>
#include <msxe370x.h>

#include <modbusclient.h>
#include <commonclient.h>

#ifdef WIN32
	#include <windows.h>
	#define sleep(a) Sleep(a*1000)
#else
	#include <unistd.h>
#endif


/** @file modbuscli.c MODBUS/UDP client sample for the MSXE3701
*
* FC16 queries are for setting things, FC3 for fetching values
*
*/

int sample_MX370x__GetLastCommandStatus(struct modbus * modbus, int *error, int *syserrno)
{
	struct GetLastCommandStatusEx_parameters_t Response;
	int ret = 0;

	memset(&Response,0,sizeof(Response));

	ret = modbus_call_GetLastCommandStatusEx(modbus, &Response);

	if ( handle_result_of_FC3_query(modbus, ret, "modbus_call_GetLastCommandStatusEx") )
		return -1;

	if (Response.ReturnValue != 0)
	{
		printf ("Return value: %i\n", Response.ReturnValue);
		*error = Response.ReturnValue;
	}

	if ((Response.Syserrno != 0) && (Response.ReturnValue == -1))
	{
		printf ("Sys errno: %i", Response.Syserrno);

		if (Response.Errstr != 0)
			printf (" ,%s", Response.Errstr);

		printf ("\n");

		*syserrno = Response.Syserrno;
	}

	return 0;
}

int sample_MX370x__TransducerSetOffsetEx(struct modbus * modbus)
{
	//variables declaration
	struct GetLastCommandStatus_parameters_t CommandStatus;
	int ret = 0;
	int i = 0;
	// Table example
	float fOffset[16] = { 1.0f, 0.1f, 0.2f, 0.3f, 0.4f, 0.5f, -1.0f, -0.1f, -0.2f, -0.3f, -0.4f, -0.5f, -0.6f, -0.7f, -0.8f, -0.9f };

	for (i=0; i<16; i++)
		printf("offset channel %d = %f\n", i, fOffset[i]);

	//Remote function call
	ret = modbus_call_MX370x__TransducerSetOffsetEx(modbus, fOffset, &CommandStatus);

	//Return value check
	return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MX370x__TransducerSetOffsetEx" );
}

//---------------------------------------------------------------------------------
int sample_MX370x__getNumberOfChannels(struct modbus * modbus, uint32_t* ChannelNumber)
{
	struct MX370x__getNumberOfChannels_parameters_t Response;
	int ret = 0;

	memset(&Response,0,sizeof(Response));

	ret = modbus_call_MX370x__getNumberOfChannels(modbus,&Response);

	if ( handle_result_of_FC3_query(modbus, ret, "modbus_call_MX370x__getNumberOfChannels") )
		return -1;

	*ChannelNumber = Response.ChannelNumber;
	printf("number of channel(s) on the module: %d\n",Response.ChannelNumber);
	return 0;
}

//---------------------------------------------------------------------------------
int sample_MX370x__getNumberOfChannelsEx(struct modbus * modbus, uint32_t* ChannelNumber)
{
	struct MX370x__getNumberOfChannelsEx_parameters_t Response;
	int ret = 0;

	memset(&Response,0,sizeof(Response));

	ret = modbus_call_MX370x__getNumberOfChannelsEx(modbus,&Response);

	if ( handle_result_of_FC3_query(modbus, ret, "modbus_call_MX370x__getNumberOfChannelsEx") )
		return -1;

	*ChannelNumber = Response.ChannelNumber;
	printf("number of channel on the module: %d\n",Response.ChannelNumber);
	return 0;
}
//---------------------------------------------------------------------------------
int sample_MX370x__TransducerGetAutoRefreshValues(struct modbus * modbus, struct msxe370x_format *format, struct msxe370x_autorefresh_parameters* Params)
{
	struct MX370x__TransducerGetAutoRefreshValues_parameters_t Response;
	int ret = 0;

	memset(&Response,0,sizeof(Response));

	ret = modbus_call_MX370x__TransducerGetAutoRefreshValues(modbus,&Response);

	if ( handle_result_of_FC3_query(modbus, ret, "modbus_call_MX370x__TransducerGetAutoRefreshValues") )
		return -1;

	msxe370x_getautorefreshvalues_printf(Params, Response.Value, Params->NbrOfChannel+1, format);
	return 0;
}
//---------------------------------------------------------------------------------
int sample_MX370x__TransducerGetAutoRefreshValuesEx(struct modbus * modbus, struct msxe370x_format *format, struct msxe370x_autorefresh_parameters* Params)
{
	struct MX370x__TransducerGetAutoRefreshValuesEx_parameters_t Response;
	int ret = 0;

	memset(&Response,0,sizeof(Response));

	ret = modbus_call_MX370x__TransducerGetAutoRefreshValuesEx(modbus,&Response);

	if ( handle_result_of_FC3_query(modbus, ret, "modbus_call_MX370x__TransducerGetAutoRefreshValuesEx") )
		return -1;

	msxe370x_getautorefreshvalues_printf(Params, Response.Value, Params->NbrOfChannel+1, format);

	return 0;
}
//---------------------------------------------------------------------------------
int sample_MX370x__TransducerInitAndStartAutoRefresh(struct modbus * modbus, struct msxe370x_autorefresh_parameters* Params, uint32_t NumberOfChannels)
{

	//variables declaration
	struct GetLastCommandStatus_parameters_t CommandStatus;
	int ret = 0;

	//Remote function call
	ret = modbus_call_MX370x__TransducerInitAndStartAutoRefresh(modbus,
			Params->TransducerSelection,
			Params->ChannelMask,
			Params->AverageMode,
			Params->AverageValue,
			Params->DivisionFactor,
			Params->TriggerAction,
			Params->HardwareTriggerCount,
			Params->HardwareTriggerFilterTime,
			Params->ByTriggerNbrOfSeqToAcquire,
			Params->DataFormat,
			Params->Option2,
			Params->Option3,
			Params->Option4,
			&CommandStatus);

	//Return value check
	return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MX370x__TransducerInitAndStartAutoRefresh" );

}
//---------------------------------------------------------------------------------
int sample_MX370x__TransducerInitAndStartAutoRefreshEx(struct modbus * modbus, struct msxe370x_autorefresh_parameters* Params, uint32_t NumberOfChannels)
{

	//variables declaration
	struct GetLastCommandStatus_parameters_t CommandStatus;
	int ret = 0;

	//Remote function call
	ret = modbus_call_MX370x__TransducerInitAndStartAutoRefreshEx(modbus,
			Params->TransducerSelection,
			Params->ChannelMask,
			Params->AverageMode,
			Params->AverageValue,
			Params->DivisionFactor,
			Params->TriggerAction,
			Params->HardwareTriggerCount,
			Params->HardwareTriggerFilterTime,
			Params->ByTriggerNbrOfSeqToAcquire,
			Params->DataFormat,
			Params->Option2,
			Params->Option3,
			Params->Option4,
			&CommandStatus);

	//Return value check
	return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MX370x__TransducerInitAndStartAutoRefreshEx" );

}
//---------------------------------------------------------------------------------
int sample_MX370x__TransducerStopAndReleaseAutoRefresh(struct modbus * modbus)
{
	//variables declaration

	struct GetLastCommandStatus_parameters_t CommandStatus;
	int ret = 0;
	uint32_t Dummy = 0;

	//Remote function call
	ret = modbus_call_MX370x__TransducerStopAndReleaseAutoRefresh(modbus,Dummy,&CommandStatus);

	return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MX370x__TransducerStopAndReleaseAutoRefresh");
}
//---------------------------------------------------------------------------------
int sample_MX370x__TransducerStopAndReleaseAutoRefreshEx(struct modbus * modbus)
{
	//variables declaration

	struct GetLastCommandStatus_parameters_t CommandStatus;
	int ret = 0;
	uint32_t Dummy = 0;

	//Remote function call
	ret = modbus_call_MX370x__TransducerStopAndReleaseAutoRefreshEx(modbus,Dummy,&CommandStatus);

	return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MX370x__TransducerStopAndReleaseAutoRefreshEx");
}
//---------------------------------------------------------------------------------
int sample_MX370x__TransducerInitAndStartSequence(struct modbus * modbus, struct msxe370x_sequence_parameters* Params, uint32_t NumberOfChannels)
{
	struct GetLastCommandStatus_parameters_t CommandStatus;
	int ret = 0;

	//Remote function call
	ret = modbus_call_MX370x__TransducerInitAndStartSequence(modbus,
			Params->TransducerSelection,
			Params->NbrOfChannel,
			Params->ChannelList,
			Params->DivisionFactor,
			Params->NbrOfSequence,
			Params->NbrMaxSequenceToTransfer,
			Params->DelayMode,
			Params->DelayTimeUnit,
			Params->DelayValue,
			Params->TriggerAction,
			Params->HardwareTriggerCount,
			Params->HardwareTriggerFilterTime,
			Params->ByTriggerNbrOfSeqToAcquire,
			Params->DataFormat,
			Params->Option2,
			Params->Option3,
			Params->Option4,
			&CommandStatus);

	//Return value check
	return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MX370x__TransducerInitAndStartSequence" );

}
//---------------------------------------------------------------------------------
int sample_MX370x__TransducerInitAndStartSequenceEx(struct modbus * modbus, struct msxe370x_sequence_parameters* Params, uint32_t NumberOfChannels)
{
	struct GetLastCommandStatus_parameters_t CommandStatus;
	int ret = 0;

	//Remote function call
	ret = modbus_call_MX370x__TransducerInitAndStartSequenceEx(modbus,
			Params->TransducerSelection,
			Params->NbrOfChannel,
			Params->ChannelList,
			Params->DivisionFactor,
			Params->NbrOfSequence,
			Params->NbrMaxSequenceToTransfer,
			Params->DelayMode,
			Params->DelayTimeUnit,
			Params->DelayValue,
			Params->TriggerAction,
			Params->HardwareTriggerCount,
			Params->HardwareTriggerFilterTime,
			Params->ByTriggerNbrOfSeqToAcquire,
			Params->DataFormat,
			Params->Option2,
			Params->Option3,
			Params->Option4,
			&CommandStatus);

	//Return value check
	return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MX370x__TransducerInitAndStartSequenceEx" );

}
//---------------------------------------------------------------------------------
int sample_MX370x__TransducerStopAndReleaseSequence(struct modbus * modbus)
{
	//variables declaration

	struct GetLastCommandStatus_parameters_t CommandStatus;
	int ret = 0;
	uint32_t Dummy = 0;

	//Remote function call
	ret = modbus_call_MX370x__TransducerStopAndReleaseSequence(modbus,Dummy,&CommandStatus);

	return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MX370x__TransducerStopAndReleaseSequence");
}
//---------------------------------------------------------------------------------
int sample_MX370x__TransducerStopAndReleaseSequenceEx(struct modbus * modbus)
{
	//variables declaration

	struct GetLastCommandStatus_parameters_t CommandStatus;
	int ret = 0;
	uint32_t Dummy = 0;

	//Remote function call
	ret = modbus_call_MX370x__TransducerStopAndReleaseSequenceEx(modbus,Dummy,&CommandStatus);

	return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MX370x__TransducerStopAndReleaseSequenceEx");
}
//---------------------------------------------------------------------------------
void print_header(void)
{
        printf ("\n\n");
        printf ("\n+---------------------------------------------------+");
        printf ("\n| MSX-E370x MODBUS Sample                           |");
        printf ("\n+---------------------------------------------------+");
        printf ("\n| Press ESC to stop an acquisition                  |");
        printf ("\n+---------------------------------------------------+");
        printf ("\n\n");
}
//---------------------------------------------------------------------------------
static const char * typestr(uint32_t type)
{
	switch(type)
	{
	case 0:
		return "half-bridge";
	case 1:
		return "LVDT";
	case 2:
		return "Knaebel";
	case 3:
		return "half-bridge Mahr";
	case 4:
		return "LVDT Mahr";
	default:
		return "UNKNOWN";
	}
}
//---------------------------------------------------------------------------------
int sample_MX370x__TransducerGetNbrOfType(struct modbus * modbus, uint32_t* NumberOfTransducerTypes)
{
	struct MX370x__TransducerGetNbrOfType_parameters_t Response;
	int ret = 0;

	memset(&Response,0,sizeof(Response));

	ret = modbus_call_MX370x__TransducerGetNbrOfType(modbus,&Response);

	if ( handle_result_of_FC3_query(modbus, ret, "modbus_call_MX370x__TransducerGetNbrOfType") )
		return -1;

	*NumberOfTransducerTypes = Response.NumberOfTransducerTypes;
	printf("number of types defined in the transducer database: %d\n",Response.NumberOfTransducerTypes);
	return 0;
}
//---------------------------------------------------------------------------------
int sample_MX370x__TransducerGetNbrOfTypeEx(struct modbus * modbus, uint32_t* NumberOfTransducerTypes)
{
	struct MX370x__TransducerGetNbrOfTypeEx_parameters_t Response;
	int ret = 0;

	memset(&Response,0,sizeof(Response));

	ret = modbus_call_MX370x__TransducerGetNbrOfTypeEx(modbus,&Response);

	if ( handle_result_of_FC3_query(modbus, ret, "modbus_call_MX370x__TransducerGetNbrOfTypeEx") )
		return -1;

	*NumberOfTransducerTypes = Response.NumberOfTransducerTypes;
	printf("number of types defined in the transducer database: %d\n",Response.NumberOfTransducerTypes);
	return 0;
}
//---------------------------------------------------------------------------------
int sample_MX370x__GetTransducerDatabaseCursor(struct modbus * modbus, uint32_t* TransducerDatabaseCursor)
{
	struct MX370x__GetTransducerDatabaseCursorEx_parameters_t Response;
	int ret = 0;

	memset(&Response,0,sizeof(Response));

	ret = modbus_call_MX370x__GetTransducerDatabaseCursorEx(modbus,&Response);

	if ( handle_result_of_FC3_query(modbus, ret, "modbus_call_MX370x__GetTransducerDatabaseCursorEx") )
		return -1;

	*TransducerDatabaseCursor = Response.TransducerDatabaseCursor;
	printf("Current transducer database cursor: %d\n",Response.TransducerDatabaseCursor);
	return 0;
}
//---------------------------------------------------------------------------------
int sample_MX370x__GetTransducerDatabaseCursorEx(struct modbus * modbus, uint32_t* TransducerDatabaseCursor)
{
	struct MX370x__GetTransducerDatabaseCursorEx_parameters_t Response;
	int ret = 0;

	memset(&Response,0,sizeof(Response));

	ret = modbus_call_MX370x__GetTransducerDatabaseCursorEx(modbus,&Response);

	if ( handle_result_of_FC3_query(modbus, ret, "modbus_call_MX370x__GetTransducerDatabaseCursorEx") )
		return -1;

	*TransducerDatabaseCursor = Response.TransducerDatabaseCursor;
	printf("Current transducer database cursor: %d\n",Response.TransducerDatabaseCursor);
	return 0;
}
//---------------------------------------------------------------------------------
int sample_MX370x__TransducerGetTypeInformation(struct modbus * modbus, struct MX370x__TransducerGetTypeInformation_parameters_t* TransducerGetTypeInformation)
{
	struct MX370x__TransducerGetTypeInformation_parameters_t Response;
	int ret = 0;

	memset(&Response,0,sizeof(Response));

	ret = modbus_call_MX370x__TransducerGetTypeInformation(modbus,&Response);

	if ( handle_result_of_FC3_query(modbus, ret, "modbus_call_MX370x__TransducerGetTypeInformation") )
		return -1;

	memcpy(TransducerGetTypeInformation, &Response, sizeof(struct MX370x__TransducerGetTypeInformation_parameters_t));

	return 0;
}
//---------------------------------------------------------------------------------
int sample_MX370x__TransducerGetTypeInformationEx(struct modbus * modbus, struct MX370x__TransducerGetTypeInformationEx_parameters_t* TransducerGetTypeInformation)
{
	struct MX370x__TransducerGetTypeInformationEx_parameters_t Response;
	int ret = 0;

	memset(&Response,0,sizeof(Response));

	ret = modbus_call_MX370x__TransducerGetTypeInformationEx(modbus,&Response);

	if ( handle_result_of_FC3_query(modbus, ret, "modbus_call_MX370x__TransducerGetTypeInformationEx") )
		return -1;

	memcpy(TransducerGetTypeInformation, &Response, sizeof(struct MX370x__TransducerGetTypeInformationEx_parameters_t));

	return 0;
}
//---------------------------------------------------------------------------------
int sample_MX370x__SetTransducerDatabaseCursor(struct modbus * modbus, uint32_t new)
{
	struct GetLastCommandStatus_parameters_t CommandStatus;
	int ret = 0;

	ret = modbus_call_MX370x__SetTransducerDatabaseCursor(modbus, new, &CommandStatus);

	return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MX370x__SetTransducerDatabaseCursor");
}
//---------------------------------------------------------------------------------
int sample_MX370x__SetTransducerDatabaseCursorEx(struct modbus * modbus, uint32_t new)
{
	struct GetLastCommandStatus_parameters_t CommandStatus;
	int ret = 0;

	ret = modbus_call_MX370x__SetTransducerDatabaseCursorEx(modbus, new, &CommandStatus);

	return handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MX370x__SetTransducerDatabaseCursorEx");
}
//---------------------------------------------------------------------------------
/** prints the content of the transducer database on STDOUT */
int dump_transducer_database(struct modbus * modbus)
{
	uint32_t NumberOfTransducerTypes = 0;

	printf ("\n-----------------------------------------------------\n");
	printf ("Printing the content of the transducer database\n");
	printf ("-----------------------------------------------------\n");

	if (sample_MX370x__TransducerGetNbrOfTypeEx(modbus, &NumberOfTransducerTypes) != 0)
		return -1;

	/* go through the transducer database */
	{
		uint32_t i;
		for(i=0; i<NumberOfTransducerTypes; i++)
		{
			uint32_t TransducerDatabaseCursor;
			struct MX370x__TransducerGetTypeInformationEx_parameters_t TransducerGetTypeInformation;

			if (sample_MX370x__SetTransducerDatabaseCursorEx(modbus, i) != 0)
				return -1;

			printf("\n");

			if (sample_MX370x__GetTransducerDatabaseCursorEx(modbus, &TransducerDatabaseCursor) != 0)
				return -1;

			if (i != TransducerDatabaseCursor)
				return -1;

			if (sample_MX370x__TransducerGetTypeInformationEx(modbus, &TransducerGetTypeInformation) != 0)
				return -1;

			printf("Selection index: %u\n", TransducerGetTypeInformation.SelectionIndex);
			printf("Name: %s\n", TransducerGetTypeInformation.Name);
			printf("Calibrated ? %s\n", ( TransducerGetTypeInformation.CalibrationStatus ? "yes" : "no" ) );
			printf("Type: %s\n", typestr(TransducerGetTypeInformation.Type));
			printf("Frequency: %u Hz\n", TransducerGetTypeInformation.Frequency);
			printf("Impedance: %u Ohm\n", TransducerGetTypeInformation.Impedance);
			printf("Nominal voltage: %f Vrms\n", TransducerGetTypeInformation.NominalVoltage);
			printf("Sensibility: %f mv/V/mm\n", TransducerGetTypeInformation.Sensibility);
			printf("Range: +/- %f mm\n", TransducerGetTypeInformation.Range);
			printf("\n");
		}
	}

	return 0;
}


/* Get the diagnosis of all the ports */
int sample_MX370x__ExtDigitalIODiagnostic(struct modbus * modbus)
{
	struct MX370x__ExtDigitalIOGetNumberOfPorts_parameters_t ResponsePortNumber;
	int ret = 0;
	unsigned long i = 0;

	printf ("-----------------------------------------------------\n");
	printf ("Ext Digital I/O Diagnosis\n");
	printf ("-----------------------------------------------------\n");

	printf("\nPress a key to continue\n");
	fflush(stdout);
	addidata_console_wait_for_input();

	/* get the number of ports */
	ret = modbus_call_MX370x__ExtDigitalIOGetNumberOfPorts(modbus, &ResponsePortNumber);
	if ( handle_result_of_FC3_query(modbus, ret, "modbus_call_MX370x__ExtDigitalIOGetNumberOfPorts") )
		return -1;

	/* get the diagnosis per port */
	for ( i = 0; i < ResponsePortNumber.ulNumber; i++ )
	{
		struct MX370x__ExtDigitalIOTestOutputsShortCircuit_parameters_t ResponseShortCircuit;
		struct MX370x__ExtDigitalIOTestOutputsPowerSupply_parameters_t ResponsePowerSupply;
		struct GetLastCommandStatus_parameters_t CommandStatus;

		/* set the port */
		ret = modbus_call_MX370x__SetDataCursor(modbus, i, &CommandStatus);
		if ( handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MX370x__SetDataCursor") )
			return -1;

		/* short-circuit diagnosis */
		ret = modbus_call_MX370x__ExtDigitalIOTestOutputsShortCircuit(modbus, &ResponseShortCircuit);
		if ( handle_result_of_FC3_query(modbus, ret, "modbus_call_MX370x__ExtDigitalIOTestOutputsShortCircuit") )
			return -1;

		/* power supply diagnosis */
		ret = modbus_call_MX370x__ExtDigitalIOTestOutputsPowerSupply(modbus, &ResponsePowerSupply);
		if ( handle_result_of_FC3_query(modbus, ret, "modbus_call_MX370x__ExtDigitalIOTestOutputsPowerSupply") )
			return -1;

		/* output result */
		printf("Short-circuit state: %d\n", ResponseShortCircuit.ulNumber);
		if (ResponseShortCircuit.ulNumber != 0)
			printf("Short-circuit detected!\n");

		printf("Power supply state: %d\n", ResponsePowerSupply.ulNumber);
		if (ResponsePowerSupply.ulNumber != 0)
			printf("No power supply detected!\n");
	}

	return 0;
}
/* Write some values on the Ext Digital I/O ports */
int sample_MX370x__ExtDigitalIOWritePort(struct modbus * modbus)
{
	struct MX370x__ExtDigitalIOGetNumberOfPorts_parameters_t ResponsePortNumber;
	int ret = 0;
	unsigned long i = 0;

	printf ("-----------------------------------------------------\n");
	printf ("Ext Digital I/O write on a port \n");
	printf ("-----------------------------------------------------\n");

	printf("\nPress a key to continue\n");
	fflush(stdout);
	addidata_console_wait_for_input();

	/* get the number of ports */
	ret = modbus_call_MX370x__ExtDigitalIOGetNumberOfPorts(modbus, &ResponsePortNumber);
	if ( handle_result_of_FC3_query(modbus, ret, "modbus_call_MX370x__ExtDigitalIOGetNumberOfPorts") )
		return -1;

	/* for each port */
	for ( i = 0; i < ResponsePortNumber.ulNumber; i++ )
	{
		struct MX370x__ExtDigitalIOGetPortDirections_parameters_t ResponsePortDirection;
		struct GetLastCommandStatus_parameters_t CommandStatus;

		/* set the port */
		ret = modbus_call_MX370x__SetDataCursor(modbus, i, &CommandStatus);
		if ( handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MX370x__SetDataCursor") )
			return -1;

		/* get the direction of the I/O of the port */
		ret = modbus_call_MX370x__ExtDigitalIOGetPortDirections(modbus, &ResponsePortDirection);
		if ( handle_result_of_FC3_query(modbus, ret, "modbus_call_MX370x__ExtDigitalIOGetPortDirections") )
			return -1;

		printf("Port direction: 0x%x \n", ResponsePortDirection.ulNumber);

		/* set all outputs to 0 */
		printf("Set all outputs to 0.\n");
		ret = modbus_call_MX370x__ExtDigitalIOWritePort(modbus, i, 0, 0, &CommandStatus);
		if ( handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MX370x__ExtDigitalIOWritePort") )
			return -1;

		/* pause */
		sleep(5);

		/* set all outputs to 1 */
		printf("Set all outputs to 1.\n");
		ret = modbus_call_MX370x__ExtDigitalIOWritePort(modbus, i, ResponsePortDirection.ulNumber, 0, &CommandStatus);
		if ( handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MX370x__ExtDigitalIOWritePort") )
			return -1;

		/* pause */
		sleep(5);

		/* set all outputs to 0 */
		printf("Set all outputs to 0.\n");
		ret = modbus_call_MX370x__ExtDigitalIOWritePort(modbus, i, 0, 0, &CommandStatus);
		if ( handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MX370x__ExtDigitalIOWritePort") )
			return -1;
	}

	return 0;
}

/* Write some values on the Ext Digital I/O channels */
int sample_MX370x__ExtDigitalIOWriteChannel(struct modbus * modbus)
{
	struct MX370x__ExtDigitalIOGetNumberOfChannels_parameters_t ResponseChannelNumber;
	int ret = 0;
	unsigned long i = 0;

	printf ("-----------------------------------------------------\n");
	printf ("Ext Digital I/O write on channels \n");
	printf ("-----------------------------------------------------\n");

	printf("\nPress a key to continue\n");
	fflush(stdout);
	addidata_console_wait_for_input();

	/* get the number of channels */
	ret = modbus_call_MX370x__ExtDigitalIOGetNumberOfChannels(modbus, &ResponseChannelNumber);
	if ( handle_result_of_FC3_query(modbus, ret, "modbus_call_MX370x__ExtDigitalIOGetNumberOfChannels") )
		return -1;

	/* for each channel -> we know that the first 15 channels are outputs */
	for (i = 16; i < ResponseChannelNumber.ulNumber; i++ )
	{
		struct GetLastCommandStatus_parameters_t CommandStatus;

		printf("Setting channel %lu \n", i);

		/* set the output to 1 */
		ret = modbus_call_MX370x__ExtDigitalIOWriteChannel(modbus, i, 1, 0, &CommandStatus);
		if ( handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MX370x__ExtDigitalIOWriteChannel") )
			return -1;

		/* pause */
		sleep(1);

		/* set the output to 0 */
		ret = modbus_call_MX370x__ExtDigitalIOWriteChannel(modbus, i, 0, 0, &CommandStatus);
		if ( handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MX370x__ExtDigitalIOWriteChannel") )
			return -1;
	}

	return 0;
}

/* Read the values of the Ext Digital I/O ports */
int sample_MX370x__ExtDigitalIOReadPort(struct modbus * modbus)
{
	struct MX370x__ExtDigitalIOGetNumberOfPorts_parameters_t ResponsePortNumber;
	int ret = 0;
	unsigned long i = 0;

	printf ("-----------------------------------------------------\n");
	printf ("Ext Digital I/O read port state \n");
	printf ("-----------------------------------------------------\n");

	printf("\nPress a key to continue\n");
	fflush(stdout);
	addidata_console_wait_for_input();

	/* get the number of ports */
	ret = modbus_call_MX370x__ExtDigitalIOGetNumberOfPorts(modbus, &ResponsePortNumber);
	if ( handle_result_of_FC3_query(modbus, ret, "modbus_call_MX370x__ExtDigitalIOGetNumberOfPorts") )
		return -1;

	/* for each port */
	for ( i = 0; i < ResponsePortNumber.ulNumber; i++ )
	{
		struct GetLastCommandStatus_parameters_t CommandStatus;
		struct MX370x__ExtDigitalIOReadPort_parameters_t ResponseReadPort;

		/* set the port */
		ret = modbus_call_MX370x__SetDataCursor(modbus, i, &CommandStatus);
		if ( handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MX370x__SetDataCursor") )
			return -1;

		/* read the state */
		ret = modbus_call_MX370x__ExtDigitalIOReadPort(modbus, &ResponseReadPort);
		if ( handle_result_of_FC3_query(modbus, ret, "modbus_call_MX370x__ExtDigitalIOReadPort") )
			return -1;

		printf("State of the port %lu : 0x%x \n", i, ResponseReadPort.ulNumber);
	}

	return 0;
}

/* Read the values of the Ext Digital I/O channels */
int sample_MX370x__ExtDigitalIOReadChannel(struct modbus * modbus)
{
	struct MX370x__ExtDigitalIOGetNumberOfChannels_parameters_t ResponseChannelNumber;
	int ret = 0;
	unsigned long i = 0;

	printf ("-----------------------------------------------------\n");
	printf ("Ext Digital I/O read channels state \n");
	printf ("-----------------------------------------------------\n");

	printf("\nPress a key to continue\n");
	fflush(stdout);
	addidata_console_wait_for_input();

	/* get the number of channels */
	ret = modbus_call_MX370x__ExtDigitalIOGetNumberOfChannels(modbus, &ResponseChannelNumber);
	if ( handle_result_of_FC3_query(modbus, ret, "modbus_call_MX370x__ExtDigitalIOGetNumberOfChannels") )
		return -1;

	/* for each channel */
	for ( i = 0; i < ResponseChannelNumber.ulNumber; i++ )
	{
		struct GetLastCommandStatus_parameters_t CommandStatus;
		struct MX370x__ExtDigitalIOReadChannel_parameters_t ResponseReadChannel;

		/* set the channel */
		ret = modbus_call_MX370x__SetDataCursor(modbus, i, &CommandStatus);
		if ( handle_result_of_FC16_query(modbus, &CommandStatus, ret, "modbus_call_MX370x__SetDataCursor") )
			return -1;

		/* read the state */
		ret = modbus_call_MX370x__ExtDigitalIOReadChannel(modbus, &ResponseReadChannel);
		if ( handle_result_of_FC3_query(modbus, ret, "modbus_call_MX370x__ExtDigitalIOReadChannel") )
			return -1;

		printf("State of the channel %lu : %d \n", i, ResponseReadChannel.ulNumber);
	}

	return 0;
}



/*--------------------------------------------------------------------------------- *
 * Diagnostic sample functions                                                      *
 * -------------------------------------------------------------------------------- */

int getSystemTransducerType(struct modbus * modbus, uint32_t *TransducerType)
{
	struct MXCommon__GetModuleTypeEx_parameters_t Result;
	int ret = 0;

	memset(&Result,0,sizeof(Result));

	ret = modbus_call_MXCommon__GetModuleTypeEx(modbus,&Result);

	if (handle_result_of_FC3_query(modbus, ret, "modbus_call_MXCommon__GetModuleTypeEx"))
		return -1;

	if (strstr((char *)Result.str, "-M MX370X{"))
		*TransducerType = 2;
	else
		*TransducerType = 1;

	return 0;
}

//---------------------------------------------------------------------------------
int sample_MX370x__TransducerInitPrimaryConnectionTest(struct modbus * modbus)
{
	struct MX370x__TransducerInitPrimaryConnectionTestEx_parameters_t Response;
	int ret = 0;
	int error = 0;
	int syserrno = 0;

	printf("\n-->sample_MX370x__TransducerInitPrimaryConnectionTest\n");

	memset(&Response,0,sizeof(Response));
	ret = modbus_call_MX370x__TransducerInitPrimaryConnectionTestEx(modbus, &Response);

	sample_MX370x__GetLastCommandStatus(modbus, &error, &syserrno);

	// Check if there is a diagnostic error
	if ((ret == -2) || (ret == 0))
	{
		switch (error)
		{
			case 0:
				printf("\t- TransducerInitPrimaryConnectionTest OK\n");
				break;

			case -1: // Internal error
				printf ("\t- Internal error\n\n");
				goto error;

			case -100: // Primary short circuit detected
				printf("\t- Primary short circuit detected, the short circuit has to be removed then the primary side rearmed\n\n");
				ret = 1;
				goto end;

			case -101: // No transducer connected
				printf ("\t- No transducer connected\n\n");
				ret = 0;
				break;

			case -103: // Functionality not available
				printf ("\t- Functionality not available\n\n");
				goto error;

			default:
				goto error;
		}
	}

end:
	return ret;

error:
	handle_result_of_FC3_query(modbus, ret, "modbus_call_MX370x__TransducerInitPrimaryConnectionTest");
	return -1;
}

//---------------------------------------------------------------------------------
int sample_MX370x__TransducerTestPrimaryConnection(struct modbus * modbus)
{
	struct MX370x__TransducerTestPrimaryConnectionEx_parameters_t Response;
	int ret = 0;
	int error = 0;
	int syserrno = 0;

	printf("\n-->sample_MX370x__TransducerTestPrimaryConnection\n");

	memset(&Response,0,sizeof(Response));
	ret = modbus_call_MX370x__TransducerTestPrimaryConnectionEx(modbus,&Response);

	sample_MX370x__GetLastCommandStatus(modbus, &error, &syserrno);

	// Check if there is a diagnostic error
	if ((ret == -2) || (ret == 0))
	{
		if (Response.ulValue == 1)
			printf("\t- All connections OK\n\n");
		else
			printf("\t- Connection open\n\n");

		switch (error)
		{
			case 0:
				printf("\t- TransducerTestPrimaryConnection OK\n");
				break;

			case -1: // Internal error
				printf ("\t- Internal error\n\n");
				goto error;

			case -100: // Primary short circuit detected
				printf("\t- Primary short circuit detected, the short circuit has to be removed then the primary side rearmed\n\n");
				ret = 1;
				goto end;

			case -101: // No transducer connected
				printf ("\t- No transducer connected\n\n");
				ret = 0;
				break;

			case -102: // Primary connection test not initialized
				printf ("\t- Primary connection test not initialized\n\n");
				ret = 0;
				break;

			case -103: // Functionality not available
				printf ("\t- Functionality not available\n\n");
				goto error;

			default:
				goto error;
		}
	}

end:
	return ret;

error:
	handle_result_of_FC3_query(modbus, ret, "modbus_call_MX370x__TransducerTestPrimaryConnection");
	return -1;
}

//---------------------------------------------------------------------------------
int sample_MX370x__TransducerTestPrimaryShortCircuit(struct modbus * modbus)
{
	struct MX370x__TransducerTestPrimaryShortCircuitEx_parameters_t Response;
	int ret = 0;

	printf("\n-->sample_MX370x__TransducerTestPrimaryShortCircuit\n");

	memset(&Response,0,sizeof(Response));

	ret = modbus_call_MX370x__TransducerTestPrimaryShortCircuitEx(modbus,&Response);

	if (handle_result_of_FC3_query(modbus, ret, "modbus_call_MX370x__TransducerTestPrimaryShortCircuit"))
		return -1;

	if (Response.ulStatus == 1)
		printf("\t- No short circuit\n\n");
	else
		printf("\t- Short circuit detected, the short circuit has to be removed then the primary side rearmed\n\n");

	return 0;
}

//---------------------------------------------------------------------------------
int sample_MX370x__TransducerRearmPrimary(struct modbus * modbus)
{
	struct MX370x__TransducerRearmPrimaryEx_parameters_t Response;

	int ret = 0;

	printf("\n-->sample_MX370x__TransducerRearmPrimary\n");

	memset(&Response,0,sizeof(Response));

	ret = modbus_call_MX370x__TransducerRearmPrimaryEx(modbus,&Response);

	if (handle_result_of_FC3_query(modbus, ret, "modbus_call_MX370x__TransducerRearmPrimary"))
		return -1;

	if (Response.ulValue == 1)
		printf("\t- Rearme OK\n\n");
	else
		printf("\t- Short circuit still here\n\n");

	return 0;
}

//---------------------------------------------------------------------------------
int sample_MX370x__TransducerTestSecondaryConnection(struct modbus * modbus, uint32_t channelIndex)
{
	/* Variables declaration */
	struct MX370x__TransducerTestSecondaryConnection0Ex_parameters_t Response0;
	struct MX370x__TransducerTestSecondaryConnection1Ex_parameters_t Response1;
	struct MX370x__TransducerTestSecondaryConnection2Ex_parameters_t Response2;
	struct MX370x__TransducerTestSecondaryConnection3Ex_parameters_t Response3;
	struct MX370x__TransducerTestSecondaryConnection4Ex_parameters_t Response4;
	struct MX370x__TransducerTestSecondaryConnection5Ex_parameters_t Response5;
	struct MX370x__TransducerTestSecondaryConnection6Ex_parameters_t Response6;
	struct MX370x__TransducerTestSecondaryConnection7Ex_parameters_t Response7;
	struct MX370x__TransducerTestSecondaryConnection8Ex_parameters_t Response8;
	struct MX370x__TransducerTestSecondaryConnection9Ex_parameters_t Response9;
	struct MX370x__TransducerTestSecondaryConnection10Ex_parameters_t Response10;
	struct MX370x__TransducerTestSecondaryConnection11Ex_parameters_t Response11;
	struct MX370x__TransducerTestSecondaryConnection12Ex_parameters_t Response12;
	struct MX370x__TransducerTestSecondaryConnection13Ex_parameters_t Response13;
	struct MX370x__TransducerTestSecondaryConnection14Ex_parameters_t Response14;
	struct MX370x__TransducerTestSecondaryConnection15Ex_parameters_t Response15;

	unsigned long int ulValue = 0;
	int ret = 0;
	int error = 0;
	int syserrno = 0;

	printf("\n-->sample_MX370x__TransducerTestSecondaryConnection\n");

	memset(&Response0,0,sizeof(Response0));
	memset(&Response1,0,sizeof(Response1));
	memset(&Response2,0,sizeof(Response2));
	memset(&Response3,0,sizeof(Response3));
	memset(&Response4,0,sizeof(Response4));
	memset(&Response5,0,sizeof(Response5));
	memset(&Response6,0,sizeof(Response6));
	memset(&Response7,0,sizeof(Response7));
	memset(&Response8,0,sizeof(Response8));
	memset(&Response9,0,sizeof(Response9));
	memset(&Response10,0,sizeof(Response10));
	memset(&Response11,0,sizeof(Response11));
	memset(&Response12,0,sizeof(Response12));
	memset(&Response13,0,sizeof(Response13));
	memset(&Response14,0,sizeof(Response14));
	memset(&Response15,0,sizeof(Response15));

	switch (channelIndex)
	{
	case 0:
		ret = modbus_call_MX370x__TransducerTestSecondaryConnection0Ex(modbus, &Response0);
		ulValue = Response0.ulValue;
		break;
	case 1:
		ret = modbus_call_MX370x__TransducerTestSecondaryConnection1Ex(modbus, &Response1);
		ulValue = Response1.ulValue;
		break;
	case 2:
		ret = modbus_call_MX370x__TransducerTestSecondaryConnection2Ex(modbus, &Response2);
		ulValue = Response2.ulValue;
		break;
	case 3:
		ret = modbus_call_MX370x__TransducerTestSecondaryConnection3Ex(modbus, &Response3);
		ulValue = Response3.ulValue;
		break;
	case 4:
		ret = modbus_call_MX370x__TransducerTestSecondaryConnection4Ex(modbus, &Response4);
		ulValue = Response4.ulValue;
		break;
	case 5:
		ret = modbus_call_MX370x__TransducerTestSecondaryConnection5Ex(modbus, &Response5);
		ulValue = Response5.ulValue;
		break;
	case 6:
		ret = modbus_call_MX370x__TransducerTestSecondaryConnection6Ex(modbus, &Response6);
		ulValue = Response6.ulValue;
		break;
	case 7:
		ret = modbus_call_MX370x__TransducerTestSecondaryConnection7Ex(modbus, &Response7);
		ulValue = Response7.ulValue;
		break;
	case 8:
		ret = modbus_call_MX370x__TransducerTestSecondaryConnection8Ex(modbus, &Response8);
		ulValue = Response8.ulValue;
		break;
	case 9:
		ret = modbus_call_MX370x__TransducerTestSecondaryConnection9Ex(modbus, &Response9);
		ulValue = Response9.ulValue;
		break;
	case 10:
		ret = modbus_call_MX370x__TransducerTestSecondaryConnection10Ex(modbus, &Response10);
		ulValue = Response10.ulValue;
		break;
	case 11:
		ret = modbus_call_MX370x__TransducerTestSecondaryConnection11Ex(modbus, &Response11);
		ulValue = Response11.ulValue;
		break;
	case 12:
		ret = modbus_call_MX370x__TransducerTestSecondaryConnection12Ex(modbus, &Response12);
		ulValue = Response12.ulValue;
		break;
	case 13:
		ret = modbus_call_MX370x__TransducerTestSecondaryConnection13Ex(modbus, &Response13);
		ulValue = Response13.ulValue;
		break;
	case 14:
		ret = modbus_call_MX370x__TransducerTestSecondaryConnection14Ex(modbus, &Response14);
		ulValue = Response14.ulValue;
		break;
	case 15:
		ret = modbus_call_MX370x__TransducerTestSecondaryConnection15Ex(modbus, &Response15);
		ulValue = Response15.ulValue;
		break;
	}

	sample_MX370x__GetLastCommandStatus(modbus, &error, &syserrno);

	// Check if there is a diagnostic error
	if ((ret == -2) || (ret == 0))
	{
		switch (error)
		{
			case 0:
				if (ulValue == 1)
					printf("\t- Connection OK\n\n");
				else
					printf("\t- Connection open\n\n");
				break;

			case -1: // Internal error
				printf ("\t- Internal error\n\n");
				goto error;

			case -100: // Primary short circuit detected
				printf ("\t- Primary short circuit detected, primary has to be rearmed\n\n");
				ret = 1;
				goto end;

			default:
				goto error;
		}
	}

end:
	return ret;

error:
	handle_result_of_FC3_query(modbus, ret, "modbus_call_MX370x__TransducerTestSecondaryConnection");
	return -1;
}

//---------------------------------------------------------------------------------
int sample_MX370x__TransducerTestSecondaryShortCircuit(struct modbus * modbus, uint32_t channelIndex)
{
	/* Variables declaration */
	struct MX370x__TransducerTestSecondaryShortCircuit0Ex_parameters_t Response0;
	struct MX370x__TransducerTestSecondaryShortCircuit1Ex_parameters_t Response1;
	struct MX370x__TransducerTestSecondaryShortCircuit2Ex_parameters_t Response2;
	struct MX370x__TransducerTestSecondaryShortCircuit3Ex_parameters_t Response3;
	struct MX370x__TransducerTestSecondaryShortCircuit4Ex_parameters_t Response4;
	struct MX370x__TransducerTestSecondaryShortCircuit5Ex_parameters_t Response5;
	struct MX370x__TransducerTestSecondaryShortCircuit6Ex_parameters_t Response6;
	struct MX370x__TransducerTestSecondaryShortCircuit7Ex_parameters_t Response7;
	struct MX370x__TransducerTestSecondaryShortCircuit8Ex_parameters_t Response8;
	struct MX370x__TransducerTestSecondaryShortCircuit9Ex_parameters_t Response9;
	struct MX370x__TransducerTestSecondaryShortCircuit10Ex_parameters_t Response10;
	struct MX370x__TransducerTestSecondaryShortCircuit11Ex_parameters_t Response11;
	struct MX370x__TransducerTestSecondaryShortCircuit12Ex_parameters_t Response12;
	struct MX370x__TransducerTestSecondaryShortCircuit13Ex_parameters_t Response13;
	struct MX370x__TransducerTestSecondaryShortCircuit14Ex_parameters_t Response14;
	struct MX370x__TransducerTestSecondaryShortCircuit15Ex_parameters_t Response15;

	int ret = 0;
	int error = 0;
	int syserrno = 0;
	unsigned long int ulValue = 0;
	printf("\n-->sample_MX370x__TransducerTestSecondaryShortCircuit\n");

	memset(&Response0,0,sizeof(Response0));
	memset(&Response1,0,sizeof(Response1));
	memset(&Response2,0,sizeof(Response2));
	memset(&Response3,0,sizeof(Response3));
	memset(&Response4,0,sizeof(Response4));
	memset(&Response5,0,sizeof(Response5));
	memset(&Response6,0,sizeof(Response6));
	memset(&Response7,0,sizeof(Response7));
	memset(&Response8,0,sizeof(Response8));
	memset(&Response9,0,sizeof(Response9));
	memset(&Response10,0,sizeof(Response10));
	memset(&Response11,0,sizeof(Response11));
	memset(&Response12,0,sizeof(Response12));
	memset(&Response13,0,sizeof(Response13));
	memset(&Response14,0,sizeof(Response14));
	memset(&Response15,0,sizeof(Response15));

	switch (channelIndex)
	{
	case 0:
		ret = modbus_call_MX370x__TransducerTestSecondaryShortCircuit0Ex(modbus, &Response0);
		ulValue = Response0.ulValue;
		break;
	case 1:
		ret = modbus_call_MX370x__TransducerTestSecondaryShortCircuit1Ex(modbus, &Response1);
		ulValue = Response1.ulValue;
		break;
	case 2:
		ret = modbus_call_MX370x__TransducerTestSecondaryShortCircuit2Ex(modbus, &Response2);
		ulValue = Response2.ulValue;
		break;
	case 3:
		ret = modbus_call_MX370x__TransducerTestSecondaryShortCircuit3Ex(modbus, &Response3);
		ulValue = Response3.ulValue;
		break;
	case 4:
		ret = modbus_call_MX370x__TransducerTestSecondaryShortCircuit4Ex(modbus, &Response4);
		ulValue = Response4.ulValue;
		break;
	case 5:
		ret = modbus_call_MX370x__TransducerTestSecondaryShortCircuit5Ex(modbus, &Response5);
		ulValue = Response5.ulValue;
		break;
	case 6:
		ret = modbus_call_MX370x__TransducerTestSecondaryShortCircuit6Ex(modbus, &Response6);
		ulValue = Response6.ulValue;
		break;
	case 7:
		ret = modbus_call_MX370x__TransducerTestSecondaryShortCircuit7Ex(modbus, &Response7);
		ulValue = Response7.ulValue;
		break;
	case 8:
		ret = modbus_call_MX370x__TransducerTestSecondaryShortCircuit8Ex(modbus, &Response8);
		ulValue = Response8.ulValue;
		break;
	case 9:
		ret = modbus_call_MX370x__TransducerTestSecondaryShortCircuit9Ex(modbus, &Response9);
		ulValue = Response9.ulValue;
		break;
	case 10:
		ret = modbus_call_MX370x__TransducerTestSecondaryShortCircuit10Ex(modbus, &Response10);
		ulValue = Response10.ulValue;
		break;
	case 11:
		ret = modbus_call_MX370x__TransducerTestSecondaryShortCircuit11Ex(modbus, &Response11);
		ulValue = Response11.ulValue;
		break;
	case 12:
		ret = modbus_call_MX370x__TransducerTestSecondaryShortCircuit12Ex(modbus, &Response12);
		ulValue = Response12.ulValue;
		break;
	case 13:
		ret = modbus_call_MX370x__TransducerTestSecondaryShortCircuit13Ex(modbus, &Response13);
		ulValue = Response13.ulValue;
		break;
	case 14:
		ret = modbus_call_MX370x__TransducerTestSecondaryShortCircuit14Ex(modbus, &Response14);
		ulValue = Response14.ulValue;
		break;
	case 15:
		ret = modbus_call_MX370x__TransducerTestSecondaryShortCircuit15Ex(modbus, &Response15);
		ulValue = Response15.ulValue;
		break;
	}

	sample_MX370x__GetLastCommandStatus(modbus, &error, &syserrno);

	// Check if there is a diagnostic error
	if ((ret == -2) || (ret == 0))
	{
		switch (error)
		{
			case 0:
				if (ulValue == 1)
					printf("\t- No short circuit\n\n");
				else
					printf("\t- Short circuit detected\n\n");
				break;

			case -1: // Internal error
				printf ("\t- Acquisition is running or possible primary short circuit detected\n\n");
				ret = 1;
				goto end;

			case -100: // Primary short circuit detected
				printf ("\t- Primary short circuit detected, primary has to be rearmed\n\n");
				ret = 1;
				goto end;

			case -103: // Functionality not available
				printf ("\t- Functionality not available\n\n");
				goto error;

			default:
				goto error;
		}
	}

end:
	return ret;

error:
	handle_result_of_FC3_query(modbus, ret, "modbus_call_TransducerTestSecondaryShortCircuit");
	return -1;
}

//---------------------------------------------------------------------------------
int sample_diagnostic(struct modbus * modbus, uint32_t ulChannelNumber)
{
	uint32_t ulChoice       = 0;
	uint32_t ulChannelIndex = 0;
	uint32_t TransducerType = 0;

	printf("\n\n--Sample diagnostic selected--\n\n");
	printf("Test if a short circuit is present on the primary side.\n"
			"If a short circuit is detected, check that:	\n"
			"\tOSC+ is not connected to the ground\n"
			"\tOSC- is not connected to the ground\n"
			"\tOSC+ is not connected to OSC-\n"
			"\tthen select \"Rearm primary\"\n\n");

	if (getSystemTransducerType(modbus, &TransducerType) != 0)
		return -1;

	if (TransducerType != 2)
	{
		if (sample_MX370x__TransducerInitPrimaryConnectionTest(modbus) < 0)
			return -1;
	}
	else
	{
		if (sample_MX370x__TransducerTestPrimaryShortCircuit(modbus) != 0)
			return -1;
	}

	do
	{
		/*
		 * If you are writing a program that makes tests automatically,
		 * you have to call functions in the following order:
		 *
		 * Init primary connection test
		 * Primary connection test
		 * Primary short circuit test (Rearm primary side, if necessary)
		 * Secondary connection test
		 * Secondary short circuit test
		 */

		printf("\n\n Tests have to be called as listed below.");
		printf("\n\n Select an action:");
		printf("\n-----------------------------------");
		if (TransducerType != 2)
		{
			printf("\n 1. Init primary connection test");
			printf("\n 2. Primary connection test");
		}

		printf("\n 3. Primary short circuit test");

		if (TransducerType != 2)
		{
			printf("\n 4. Secondary connection test");
			printf("\n 5. Secondary short circuit test");
		}

		printf("\n 6. Rearm primary side");
		printf("\n 7. Exit");
		printf("\n\n Your choice: ");
		scanf("%lu", (unsigned long *)&ulChoice);
		printf("\nchoice = %lu\n", (unsigned long)ulChoice);

		if ((TransducerType == 2) && (ulChoice < 5))
			continue;

		switch (ulChoice)
		{
		case 1:
			if (sample_MX370x__TransducerInitPrimaryConnectionTest(modbus) < 0)
				return -1;
			break;
		case 2:
			if (sample_MX370x__TransducerTestPrimaryConnection(modbus) < 0)
				return -1;
			break;
		case 3:
			if (sample_MX370x__TransducerTestPrimaryShortCircuit(modbus) != 0)
				return -1;
			break;
		case 4:
			/* Test if the selected channel has a short circuit on the secondary side.
			 * A short circuit means that the channel is connected to the ground.
			 * No ream is necessary after a secondary side short circuit.
			 */
			do
			{
				printf("\n Select the channel to test (0 to %lu):",(unsigned long)ulChannelNumber-1);
				scanf("%lu",(unsigned long *)&ulChannelIndex);
			}
			while (ulChannelIndex >= ulChannelNumber);

			if (sample_MX370x__TransducerTestSecondaryConnection(modbus, ulChannelIndex) < 0)
				return -1;
			break;
		case 5:
			/* Test if the selected channel has a short circuit on the secondary side.
			 * A short circuit means that the channel is connected to the ground.
			 * No ream is necessary after a secondary side short circuit.
			 */
			do
			{
				printf("\n Select the channel to test (0 to %lu):",(unsigned long)ulChannelNumber-1);
				scanf("%lu",(unsigned long *)&ulChannelIndex);
			}
			while (ulChannelIndex >= ulChannelNumber);

			if (sample_MX370x__TransducerTestSecondaryShortCircuit(modbus, ulChannelIndex) < 0)
				return -1;
			break;
		case 6:
			if (sample_MX370x__TransducerRearmPrimary(modbus) != 0)
				return -1;
			break;
		}
	}
	while (ulChoice != 7);

	return 0;
}

//---------------------------------------------------------------------------------
int main(int argc, char** argv)
{
	struct modbus modbus;
	uint32_t NumberOfChannels;
	char address[100];

	if (argc != 1)
	{
		printf("%s does not take a parameter\n",argv[0]);
		exit(1);
	}

	print_header();
	modbus_init(&modbus);

	/* ask for network address */
	while( addidata_network_ask("module IP address?>", address) )
		printf("error while parsing IP address (CTRL+C to end)\n");

	switch( modbus_connect(&modbus, address, SOCK_DGRAM, 0) )
	{
		case 0: /* success */
			break;

		case -1: /* system error */
			addidata_network_perror("modbus_connect");
			goto sample_end;

		case -2: /* parameter is incorrect */
			printf("modbus_connect: incorrect parameter\n");
			goto sample_end;

		case -3: /* incorrect address */
			printf("incorrect address\n");
			goto sample_end;
	}


	if (sample_MX370x__ExtDigitalIODiagnostic(&modbus) != 0)
		goto sample_end;

	if (sample_MX370x__ExtDigitalIOWritePort(&modbus) != 0)
		goto sample_end;

	if (sample_MX370x__ExtDigitalIOWriteChannel(&modbus) != 0)
		goto sample_end;

	if (sample_MX370x__ExtDigitalIOReadPort(&modbus) != 0)
		goto sample_end;

	if (sample_MX370x__ExtDigitalIOReadChannel(&modbus) != 0)
		goto sample_end;


	printf ("-----------------------------------------------------\n");
	printf ("Calling some common functions\n");
	printf ("-----------------------------------------------------\n");

	if (sample_MXCommon__GetModuleTypeEx(&modbus) != 0)
		goto sample_end;

	if (sample_MXCommon__GetTimeEx(&modbus) != 0)
		goto sample_end;

	printf ("-----------------------------------------------------\n");
	printf ("Calling system specific functions\n");
	printf ("-----------------------------------------------------\n");

	if (sample_MX370x__getNumberOfChannelsEx(&modbus, &NumberOfChannels) != 0)
		goto sample_end;

	if (dump_transducer_database(&modbus) != 0)
		goto sample_end;

	if (sample_MXCommon__SetHardwareTriggerFilterTimeEx(&modbus, 1) != 0)
		goto sample_end;

	{
		int AcquisitionType;
		/* set formats for output */
		struct msxe370x_format format;

		format.counter = "counter:%d ";
		format.dataheader = "%u:";
		format.hwtrigger = "HW trigger info: 0x%X\t";
		format.digital = "%u";
		format.analog = "%f";
		format.datasep = " ";
		format.recordterm = "\n";

		/* general questions */
		AcquisitionType = addidata_console_get_unsigned_integer("Select the action to do:\n0: Autorefresh\n1: Sequence\n2: SetOffset\n3: Diagnostic\n", 0,3);

		printf("\n");

		switch(AcquisitionType)
		{
		case 0: /* autorefresh */
		{
			int NumberOfAcquisition;
			int UseModbusFunction=0;
			struct msxe370x_autorefresh_parameters Params;

			NumberOfAcquisition = addidata_console_get_unsigned_integer("Number of acquisition ", 1,1000000);
			Params.TransducerSelection = addidata_console_get_unsigned_integer("Transducer selection index", 0,255);

			msxe370x_ask_autorefresh_parameters(&Params, NumberOfChannels);
			msxe370x_ask_autorefresh_format(&Params);

			printf("\n0\tFetch data with MODBUS\n");
			printf("1\tGet data via the dataserver\n");
			UseModbusFunction = !addidata_console_get_yn("Choice?");

			if (sample_MX370x__TransducerInitAndStartAutoRefreshEx(&modbus, &Params, NumberOfChannels) != 0)
				goto sample_end;

			if (UseModbusFunction)
			{
				int i;
				key_thread_handle_t kt;
				unsigned int stop = 0;
				wait_key_input_thread_parameters_t ktparams;
				ktparams.character = 27; // ESC
				ktparams.flag = &stop;

				kt = addidata_start_key_thread(&ktparams);
				{
					for(i=0;i<NumberOfAcquisition;i++)
					{
						if (sample_MX370x__TransducerGetAutoRefreshValuesEx(&modbus, &format, &Params) != 0)
							goto sample_end;

						if (stop)
							break;
					}
				}
				addidata_stop_key_thread(kt);
			}
			else
			{
				int fd;
				uint32_t* data;
				unsigned int count = msxe370x_autorefresh_compute_fields(&Params);
				data = malloc(count*4);
				if (!data)
				{
					perror("malloc");
					goto sample_end;
				}

				/* connect to data server */
				switch(addidata_dataclient_tcp_connect(address, 0, &fd))
				{
					case 0: /* OK */
						break;

					case -1:
						addidata_network_perror("socket");
						goto sample_end;

					case -2:
						addidata_network_perror("connect");
						goto sample_end;

					case -3:
						printf("error while parsing address\n");
						goto sample_end;

					case -4:
						addidata_network_perror("WSACleanup() failed");
						goto sample_end;
				}
				/* set timeout */
				{
					struct timeval timeout = MODBUS_NETWORK_TIMEOUT;
					if ( addidata_network_set_socket_timeout(fd, timeout) )
					{
						addidata_network_perror("setting timeout");
						goto sample_end;
					}
				}
				{
					int i;
					key_thread_handle_t kt;
					unsigned int stop = 0;
					wait_key_input_thread_parameters_t ktparams;
					ktparams.character = 27; // ESC
					ktparams.flag = &stop;
					kt = addidata_start_key_thread(&ktparams);
					for(i=0;i<NumberOfAcquisition;i++)
					{
						if ( -1 == addidata_dataclient_recvall(fd, data, count) )
						{
							addidata_network_perror("addidata_dataclient_recvall");
							sample_MX370x__TransducerStopAndReleaseAutoRefreshEx(&modbus);
							goto sample_end;
						}
						msxe370x_autorefresh_printf(&Params, data, count, &format);
						if (stop)
							break;
					}
					addidata_stop_key_thread(kt);
				}
				addidata_dataclient_disconnect(fd);
			}
			if (sample_MX370x__TransducerStopAndReleaseAutoRefreshEx(&modbus) != 0)
				goto sample_end;

			break;
		}
		case 1: /* sequence */
			{
				struct msxe370x_sequence_parameters Params;
				Params.TransducerSelection = addidata_console_get_unsigned_integer("Transducer selection index", 0,255);
				msxe370x_ask_sequence_parameters(&Params, NumberOfChannels);
				msxe370x_ask_sequence_format(&Params);

				/* fetch start acquisition, fetch data */
				{
					int fd;
					uint32_t* data;
					unsigned int count = msxe370x_sequence_compute_fields(&Params);
					printf("expected %d fields\n",count);
					data = malloc(count*4);
					if (!data)
					{
						perror("malloc");
						goto sample_end;
					}

					/* connect to data server */
					switch(addidata_dataclient_tcp_connect(address, 0, &fd))
					{
						case 0: /* OK */
							break;

						case -1:
							addidata_network_perror("socket");
							goto sample_end;

						case -2:
							addidata_network_perror("connect");
							goto sample_end;

						case -3:
							printf("error while parsing address\n");
							goto sample_end;

						case -4:
							addidata_network_perror("WSACleanup() failed");
							goto sample_end;
					}
					/* set timeout */
					{
						struct timeval timeout = MODBUS_NETWORK_TIMEOUT;
						if ( addidata_network_set_socket_timeout(fd, timeout) )
						{
							addidata_network_perror("setting timeout");
							goto sample_end;
						}
					}
					/* start acquisition */
					if (sample_MX370x__TransducerInitAndStartSequenceEx(&modbus, &Params, NumberOfChannels) != 0)
						goto sample_end;

					{
						key_thread_handle_t kt;
						unsigned int i = 0;
						unsigned int stop = 0;
						wait_key_input_thread_parameters_t ktparams;
						ktparams.character = 27; // ESC
						ktparams.flag = &stop;
						kt = addidata_start_key_thread(&ktparams);
						while(1)
						{
							if ( -1 == addidata_dataclient_recvall(fd, data, count) )
							{
								addidata_network_perror("fetch");
								sample_MX370x__TransducerStopAndReleaseSequenceEx(&modbus);
								goto sample_end;
							}
							msxe370x_sequence_printf(&Params, data, count, &format);
							i++;
							if ( (Params.NbrOfSequence!=0) && (i==Params.NbrOfSequence) )
								break;

							if (stop)
								break;
						}
						addidata_stop_key_thread(kt);
					}
					addidata_dataclient_disconnect(fd);
				}

				/* stop acquisition */

				if (sample_MX370x__TransducerStopAndReleaseSequenceEx(&modbus) != 0)
					goto sample_end;

				break;
			}

			case 2:
			{
				if (sample_MX370x__TransducerSetOffsetEx(&modbus) != 0)
					goto sample_end;

				break;
			}
			case 3:
			{
				if (sample_diagnostic(&modbus, NumberOfChannels) != 0)
					goto sample_end;

				break;
			}
		}
	}

sample_end:
	if (modbus_shutdown(&modbus) != 0)
		printf("\nfailed to free modbus\n");

	printf("\npress a key to end this sample\n");
	fflush(stdout);
	addidata_console_wait_for_input();

	return 0;
}
