#ifndef _THREAD_SAFE
#define _THREAD_SAFE
#endif

#include <string.h>
#include <stdio.h>
#include <time.h>

#ifdef WIN32
	#include <stdint_compat.h>
	typedef int socklen_t;
	#define __INLINE__ __inline
	#define __FUNCTION__ "Function"
	#define MSG_WAITALL 0
	#define MSG_NOSIGNAL 0
	#define MSG_EOR 0
	#define MSG_DONTWAIT 0
	#include <winsock2.h>
	#include <conio.h>
	#define SHUT_RDWR SD_BOTH
#else
	#include <unistd.h>
	#include <sys/types.h>
	#include <netinet/in.h>
	#include <arpa/inet.h>
	#include <netdb.h>
	#include <sys/socket.h>
	#include <errno.h>
#endif

#include <addidata_dataclient.h>
#include <addidata_network.h>

//---------------------------------------------------------------------------------
int addidata_dataclient_tcp_connect(const char* name, unsigned int port, int* fd)
{
	struct sockaddr_in addr; /* converted inet address */
	int tmpfd = -1;

	if ( addidata_network_addrparse(name, &addr) )
		return -3;

	if (0==port)
		port = DEFAULT_DATASERVER_PORT;

	if ( -1 == ((tmpfd) = socket(AF_INET, SOCK_STREAM, 0)) )
	{
		return -1;
	}

	/* network addresses are always in network order */
	addr.sin_port = htons((uint16_t) port);

	if ( connect(tmpfd,(struct sockaddr*) &(addr), sizeof(struct sockaddr_in)) < 0)
	{
		return -2;
	}
	*fd=tmpfd;
	return 0;
}
//---------------------------------------------------------------------------------
int addidata_dataclient_recv(int fd, uint32_t data[], int count, int flags)
{
	int ret = -1;
	int toread = count*4;
	ret = recv(fd, (void*)data, toread, flags);

	if(-1==ret)
	{
#ifdef WIN32
		if (WSAGetLastError() == 10040) /* 10040 is not a true error */
		{
			ret = toread;
		}
#endif
	}
	return ret;
}
//---------------------------------------------------------------------------------
int addidata_dataclient_recvall(int fd, uint32_t data[], int count)
{
	char* tmpbuff = (char*)data; /* eases pointer algorithmic */
	unsigned long int total=0; /* total actually read */
	unsigned long int nbbytes=count*4; /* complete number of bytes to read from socket */
	while(1)
	{
		int ret;
		int toread = nbbytes-total; /* number of bytes to read this time */

		ret = recv(fd, tmpbuff, toread, MSG_WAITALL);
		if(ret==-1)
		{
#ifdef WIN32
			if (WSAGetLastError() == 10040) /* 10040 is not a true error */
			{
				ret = toread;
			}
			else
#endif
				return -1;
		}
		if(ret==0)
			return 0;
		total += ret;
		if(total==nbbytes)
			return nbbytes;
		tmpbuff += total;
	}
}
//---------------------------------------------------------------------------------
int addidata_dataclient_recvall_timeout(int fd, uint32_t data[], int count, struct timeval* timeout)
{
	// Create a copy of the timeout (cause select modifies the pointer)
	struct timeval timeout_select;

	// Create the select structure
	fd_set socketSelect;
	FD_ZERO(&socketSelect);
	FD_SET(fd,&socketSelect);

	// Create a copy of the timeout (cause select modifies the pointer)
	timeout_select.tv_sec = timeout->tv_sec;
	timeout_select.tv_usec = timeout->tv_usec;

	switch(select(fd+1, &socketSelect,NULL,NULL,&timeout_select)){
		case 0:
			return 0;
			break;
		case -1:
			// error
			return -1;
			break;
		default:
			// Got data in the socket reception
			return (addidata_dataclient_recvall(fd,(void*)data,count));
			break;
	}
	return -1;
}
//---------------------------------------------------------------------------------
int addidata_dataclient_recvall_ifavailable(int fd,uint32_t data[], int count)
{
	int ret = recv(fd, (void*)data, count * 4, MSG_PEEK | MSG_DONTWAIT);

	if(ret==-1)
	{
#ifndef WIN32
		if (errno != EAGAIN)
			return -1;
		else
			return 0; // no data available
#endif
	}
	else if (ret > 0)
	{
		if (ret >= (count *4))
		{
			return (addidata_dataclient_recvall(fd, (void*)data, count));
		}
	}

	return ret;
}
//---------------------------------------------------------------------------------
int addidata_dataclient_disconnect(int fd)
{
	shutdown(fd,SHUT_RDWR);
#ifdef WIN32
	closesocket(fd);
#else
	close(fd);
#endif
	return 0;
}
//---------------------------------------------------------------------------------
static int is_a_format(char c)
{
	switch(c)
	{
	case 'f':
	case 'u':
	case 'd':
	case 'x':
	case 'X':
	case 'y':
	case '%':
		return 1;
	default:
		return 0;
	}
}
//---------------------------------------------------------------------------------
int addidata_dataclient_fmtcount(const char *format)
{
	int count = 0;
	while(*format)
	{
		switch(*format)
		{
		case '%':
			++(format);
			if (! is_a_format(*format))
				return -1; /* unknown format */
			count++;
			break;
		default:
			break;
		}
		++(format);
	}
	return count;
}
//---------------------------------------------------------------------------------
/* returns number of conversion done -
 * -1 if parse error*/
static int wrtformat(FILE *stream, char c, uint32_t data[], int i)
{
	switch(c)
	{
	case 'f':
		fprintf(stream,"%f",(float)data[i]);
		return 1;
	case 'u':
		fprintf(stream,"%u",data[i]);
		return 1;
	case 'd':
		fprintf(stream,"%d",data[i]);
		return 1;
	case 'x':
		fprintf(stream,"%x",data[i]);
		return 1;
	case 'X':
		fprintf(stream,"%X",data[i]);
		return 1;
	case 'y': /* skip */
		return 1;
	case '%':
		fputc('%',stream);
		return 0;
	default: /* unknown format */
		return -1;
	}
}
//---------------------------------------------------------------------------------
int addidata_dataclient_fprintf(FILE *stream, const char *format, uint32_t data[], int count)
{
	const char* tmpptr = format;
	int i=0;
	while(1)
	{
		switch(*tmpptr)
		{
			case '\0': /* end of string */
				/* stop if there is no more values in "data" */
				if (i>=count)
					return 1;  /* number of converted values */
				 /* else do it again */
				tmpptr = format;
			case '%':
			{
				int ret;
				/* overflow error when there is more format than "count" */
				if (i>=count && *(tmpptr+1)!='%')
					return -2;
				++(tmpptr);
				ret = wrtformat(stream, *tmpptr, data, i);
				if(-1==ret) /* unknown format */
					return i;
				i+=ret;
			}
			break;
			default:
				fputc(*tmpptr,stream);
		}
		++(tmpptr);
	}
}
//---------------------------------------------------------------------------------
int addidata_dataclient_printf(const char *format, uint32_t data[], int count)
{
	return addidata_dataclient_fprintf(stdout, format, data, count);
}
