#include <unistd.h>
#include <stdio.h>
#include <stdbool.h>
#include <string.h>
#include <errno.h>
#include <MX370xStub.h> // gSOAP
#include <stdio.h>
#include <termios.h>
#include <unistd.h>

#include "utils.h"

//-----------------------------------------------------------------------------------------
void mx_soap_init(struct soap * soapenv)
{
	/* set the KEEPALIVE flag on the TCP/IP socket Ref: libc - 16.12 Socket Options */
	soap_init2(soapenv, SOAP_IO_KEEPALIVE, SOAP_IO_KEEPALIVE);
	/* set a timeout in seconds */
	soapenv-> send_timeout = 1;
	soapenv-> recv_timeout = 1;
	soapenv-> accept_timeout = 1;
}

//-----------------------------------------------------------------------------------------
int parse_number_of_channels (char* str, unsigned int * number_of_channels)
{
	char* firstdash = NULL;
	char* seconddash = NULL;

	/* check presence of the substring "UNKNOWN" indicating a not yet configured module */
	if (strstr(str,"UNKNOWN"))
	{
		fprintf(stderr,"! Remote module not configured\n");
		return -1;
	}

	/* check for DIO system */
	if ( strstr(str, "DIO"))
	{
		char * typepattern = NULL;
		char channelnb[11] = {0};
		int i = 0;
		int index = 0;

		/* str is like: MSX-E3701-DIO-HB-XX or MSX-E3701-DIO-LVDT-XX*/
		/* find the LVDT or HB patter */
		typepattern = strstr(str, "LVDT");
		if ( !typepattern )
			typepattern = strstr(str,"HB");
		if ( !typepattern)
		{
			fprintf(stderr,"! incorrect type string : %s\n", str);
			return -1;
		}

		/* get the first '-' separator */
		firstdash = strstr(typepattern, "-");
		if ( ! firstdash )
		{
			fprintf(stderr,"! incorrect type string : %s\n", str);
			return -1;
		}

		/* now we can extract the number of channels */
		for ( i = 1; (i < strlen(firstdash)) && (index < 10); i++)
		{
			if ( (firstdash[i] < '0') || (firstdash[i] > '9'))
				break;

			channelnb[index] = firstdash[i];
			index++;
		}

		/* check */
		if ( index == 0)
		{
			fprintf(stderr,"! incorrect type string : %s\n", str);
			return -1;
		}

		/* convert and return */
		channelnb[index] = '\0';
		*number_of_channels = atol(channelnb);
		return 0;
	}

	/* find first '-' separator in string */
	if ( (! (firstdash = index(str, '-' ) ) )  )
	{
		fprintf(stderr,"! incorrect type string\n");
		return -1;
	}

	/* find second '-' separator in string */
	if ( ! (seconddash = index(firstdash+1, '-' ) ) )
	{
		fprintf(stderr,"! incorrect type string\n");
		return -1;
	}

	*seconddash='\0';

	switch ( (*number_of_channels = atol(firstdash+1) ) )
	{
		case 4:
		case 8:
		case 16:
			return 0; // OK
			break;
		default:
			break;
	}

	return -1;
}
//-------------------------------------------------------------------
/* get_address - get numeric or symbolic string adress
 *
 * param[in] target_name a string containing a IP adress or a symbolic name
 * param[out] to traduced inet address structure
 */
//-------------------------------------------------------------------
static int str_to_inetaddress(struct sockaddr_in * to, const char* target_name)
{
	bool found = false;
	struct hostent* hostinfo;
	struct netent* netinfo;

	/* first initialize the type of address */
	to->sin_family = AF_INET;

	// try as doted network address
	found = (bool) inet_aton(target_name, &(to->sin_addr));

	//else let try symbolic host name
	if (!found)
	{
		found = ((hostinfo = gethostbyname(target_name)) != NULL);
		if (found)
		{
				to->sin_addr = *(struct in_addr *) hostinfo->h_addr;
				return 0;
		}
	}

	// else symbolic network name ?
	if (!found)
	{
		found = ((netinfo = getnetbyname(target_name)) != NULL);
		if (found)
		{
				to->sin_addr.s_addr = ntohl(netinfo->n_net);
				return 0;
		}
	}

	if (!found)
	{
		errno=EINVAL;
		return -1;
	}

	return 0;
}
//-------------------------------------------------------------------
/* strtoport - string to port number.
 *
 * @note MUST be called only in a monothreaded code point.
 *
 * @return 0 on success, -1 on error ( see errno variable )
*/
static int strtoport (const char* str, u_int16_t * port)
{
		long int tmpint;
		char* strptr;

		tmpint = strtol(str,&strptr,10);

		/*
		 * warning: due to the use of the shared errno variable, this code must
		 * be called only in a monothreaded code point.
		 */
		if (errno == ERANGE)
		{
			if (tmpint==LONG_MIN)
			{
				errno=EINVAL;
				return -1;
			}

			if (tmpint==LONG_MAX)
			{
				errno=EINVAL;
				return -1;
			}
		}

		if (str == strptr)
		{
			errno=EINVAL;
			return -1;
		}

		if (tmpint > 65535)
		{
			errno=EINVAL;
			return -1;
		}

		*port = (u_int16_t) tmpint;
		return 0;

}
//-------------------------------------------------------------------
int datasrv_connect(const char* name)
{
	struct sockaddr_in myip;
	u_int16_t port = 0;
	memset(&myip,0,sizeof(myip));

	int fd=-1;

	/* convert the string as a BSD network address structure */
	if (str_to_inetaddress( &myip,name) )
	{
		perror(__FUNCTION__);
		abort();
	}

	/* same as above for port number */
	if (strtoport("8989",&port))
	{
		perror(__FUNCTION__);
		abort();
	}
	myip.sin_port = htons(port);

	/* create a new socket */
	if ( (fd=socket(AF_INET,SOCK_STREAM,0) ) < 0)
	{
		perror(__FUNCTION__);
		abort();
	}

	/* specify a timeout of one second for this socket
	 *
	 * This is not much. You may want to extend this value
	 *
	 */
	{
		struct timeval io_timeout;
		io_timeout.tv_sec = 0; /* second */
		io_timeout.tv_usec = 0; /* microseconds */

		/* timeout for receive buffer */
		if ( setsockopt(fd,SOL_SOCKET,SO_RCVTIMEO,&(io_timeout),sizeof(struct timeval)) )
		{
			perror("setsockopt:SO_RCVTIMEO");
			abort();
		}
		/* timeout for send buffer */
	    if ( setsockopt(fd,SOL_SOCKET,SO_SNDTIMEO,&(io_timeout),sizeof(struct timeval)) )
		{
			perror("setsockopt:SO_SNDTIMEO");
			abort();
		}
	}
	/* try to connect to the server */
	if ( connect(fd,(struct sockaddr*) &myip, sizeof(myip)) < 0)
	{
		perror(__FUNCTION__);
		abort();
	}
	printf("data: connected to %s\n",name);
	return fd;
}
//-----------------------------------------------------------------------------------------
void __assert_gsoap(struct soap* soapenv, bool flag, const char * _file, const char* _function, int _line)
{
	if (!flag)
	{
		fprintf(stderr,"SOAP error at %s:%s:%d\n message: ",_file,_function,_line);
		soap_print_fault(soapenv, stderr);
		mx_soap_clean(soapenv);
		abort();
	}
}
//-----------------------------------------------------------------------------------------
void __assert_soap_command(struct soap * soapenv, struct DefaultResponse* Response, const char * _file, const char* _function, int _line)
{
	if (Response->iReturnValue)
	{
		/* retrieve the descriptive string associated with the error */
		struct MXCommon__ByteArrayResponse Response1;
		assert_gsoap(soapenv,soap_call_MXCommon__Strerror(soapenv,NULL, NULL,Response->syserrno, &Response1) );
		fprintf(stderr,"remote command error at %s:%s:%d\nmessage: %s\n",_file,_function,_line,Response1.sArray.__ptr);
		abort();
	}
}


//-----------------------------------------------------------------------------------------
int getche(void)
   {
   struct termios oldt, newt;
   int ch;

   tcgetattr (STDIN_FILENO, &oldt);
   memcpy (&newt, &oldt, sizeof (newt));
   newt.c_lflag &= ~(ICANON);
   tcsetattr (STDIN_FILENO, TCSANOW, &newt);
   ch = getchar();
   tcsetattr (STDIN_FILENO, TCSANOW, &oldt);

   return (ch);
   }

//-----------------------------------------------------------------------------------------
int getch(void)
   {
   struct termios oldt, newt;
   int ch;

   tcgetattr (STDIN_FILENO, &oldt);
   memcpy (&newt, &oldt, sizeof (newt));
   newt.c_lflag &= ~(ICANON | ECHO);
   tcsetattr (STDIN_FILENO, TCSANOW, &newt);
   ch = getchar();
   tcsetattr (STDIN_FILENO, TCSANOW, &oldt);

   return (ch);
   }

//-----------------------------------------------------------------------------------------
int kbhit (void)
   {
   struct termios oldt, newt;
   struct timeval tv;
   fd_set         read_fd;
   int            status;

   tcgetattr (STDIN_FILENO, &oldt);
   memcpy (&newt, &oldt, sizeof (newt));
   newt.c_lflag &= ~(ICANON | ECHO);
   tcsetattr (STDIN_FILENO, TCSANOW, &newt);

   tv.tv_sec  = 0;
   tv.tv_usec = 0;
   FD_ZERO (&read_fd);
   FD_SET  (0, &read_fd);

   status = select(1, &read_fd, NULL, NULL, &tv);

   tcsetattr (STDIN_FILENO, TCSANOW, &oldt);

   if (status < 0)
      {
      return (0);
      }
   else
      {
      return (status);
      }
   }

/** Display a timeval struct in human string
 *
 * Convert and save date and time element
 * of a timeval struct in comprehensible outputs:
 * day/month/years hour:min.:sec.:msec.:usec.
 */
char *DisplayTimeStamp (unsigned long TimeStampSec, unsigned long TimeStampUSec)
   {
   struct tm *tm_ptr;
   time_t useconds, mseconds;
   static char cWrite [300];

   tm_ptr = localtime ((time_t *) &TimeStampSec);
   useconds = TimeStampUSec;
   mseconds = useconds/1000;
   useconds -= (mseconds * 1000);
   printf ("%02d/%02d/%04d-%02d:%02d:%02d:%03ld:%03ld ", tm_ptr->tm_mday, (tm_ptr->tm_mon+1), (tm_ptr->tm_year + 1900), tm_ptr->tm_hour, tm_ptr->tm_min, tm_ptr->tm_sec, mseconds, useconds);
   sprintf(cWrite,"%02d/%02d/%04d-%02d:%02d:%02d:%03ld:%03ld ", tm_ptr->tm_mday, (tm_ptr->tm_mon+1), (tm_ptr->tm_year + 1900), tm_ptr->tm_hour, tm_ptr->tm_min, tm_ptr->tm_sec, mseconds, useconds);

   return cWrite;
   }
