using System;
using System.Collections.Generic;
using System.Text;

namespace Diagnostic
{
    public class MSXE3701
    {
        /// <summary>
        /// Web reference to the system
        /// Enables access on it
        /// </summary>
        private MX370xWeb.MX370x sys;

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="ip">IP address of the system</param>
        public MSXE3701(String ip)
        {
            // Construct the web reference
            this.sys = new MX370xWeb.MX370x();
            try
            {
                this.sys.Url = "http://" + ip + ":5555";
                this.sys.Timeout = 2000;
            }
            catch (Exception)
            {
                this.sys = null;
            }
        }

        /// <summary>
        /// Get the number of external digital port available
        /// </summary>
        /// <param name="portNumber"></param>
        /// <param name="error"></param>
        /// <returns></returns>
        public int ExtDigitalIOGetNumberOfPorts(out ulong portNumber, ref string error)
        {
            // initialisation
            portNumber = 0;
            error = string.Empty;

            // Check
            if (this.sys == null)
            {
                error = "System is null. Check the IP address.";
                return -3;
            }

            try
            {
                MX370xWeb.DefaultResponse response = this.sys.ExtDigitalIOGetNumberOfPorts(0, out portNumber);
                if (response.iReturnValue != 0)
                {
                    error = "ExtDigitalIOGetNumberOfPorts error: " + response.iReturnValue.ToString();
                    return -1;
                }
            }
            catch (Exception ex)
            {
                error = "ExtDigitalIOGetNumberOfPorts exception: " + ex.Message;
                return -2;
            }
            return 0;
        }

        /// <summary>
        /// Get the status of the external power supply for a given port
        /// </summary>
        /// <param name="port"></param>
        /// <param name="status"></param>
        /// <param name="error"></param>
        /// <returns></returns>
        public int ExtDigitalIOTestOutputsPowerSupply(ulong port, out ulong status, ref string error)
        {
            // initialisation
            status = 0;
            error = string.Empty;

            // Check
            if (this.sys == null)
            {
                error = "System is null. Check the IP address.";
                return -3;
            }

            try
            {
                MX370xWeb.DefaultResponse response = this.sys.ExtDigitalIOTestOutputsPowerSupply(port, 0, out status);
                if (response.iReturnValue != 0)
                {
                    error = "ExtDigitalIOTestOutputsPowerSupply error: " + response.iReturnValue.ToString();
                    return -1;
                }
            }
            catch (Exception ex)
            {
                error = "ExtDigitalIOTestOutputsPowerSupply exception: " + ex.Message;
                return -2;
            }
            return 0;
        }

        /// <summary>
        /// Get the status of the short-circuit
        /// </summary>
        /// <param name="port"></param>
        /// <param name="status"></param>
        /// <param name="error"></param>
        /// <returns></returns>
        public int ExtDigitalIOTestOutputsShortCircuit(ulong port, out ulong status, ref string error)
        {
            // initialisation
            status = 0;
            error = string.Empty;

            // Check
            if (this.sys == null)
            {
                error = "System is null. Check the IP address.";
                return -3;
            }

            try
            {
                MX370xWeb.DefaultResponse response = this.sys.ExtDigitalIOTestOutputsShortCircuit(port, 0, out status);
                if (response.iReturnValue != 0)
                {
                    error = "ExtDigitalIOTestOutputsShortCircuit error: " + response.iReturnValue.ToString();
                    return -1;
                }
            }
            catch (Exception ex)
            {
                error = "ExtDigitalIOTestOutputsShortCircuit exception: " + ex.Message;
                return -2;
            }
            return 0;
        }
    }
}
