using System;
using System.Collections.Generic;
using System.Text;

namespace Filter
{
    public class MSXE3701
    {
        /// <summary>
        /// Web reference to the system
        /// Enables access on it
        /// </summary>
        private MX370xWeb.MX370x sys;

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="ip">IP address of the system</param>
        public MSXE3701(String ip)
        {
            // Construct the web reference
            this.sys = new MX370xWeb.MX370x();
            try
            {
                this.sys.Url = "http://" + ip + ":5555";
                this.sys.Timeout = 2000;
            }
            catch (Exception)
            {
                this.sys = null;
            }
        }

        /// <summary>
        /// Get the number of external digital port available
        /// </summary>
        /// <param name="portNumber"></param>
        /// <param name="error"></param>
        /// <returns></returns>
        public int ExtDigitalIOGetNumberOfPorts(out ulong portNumber, ref string error)
        {
            // initialisation
            portNumber = 0;
            error = string.Empty;

            // Check
            if (this.sys == null)
            {
                error = "System is null. Check the IP address.";
                return -3;
            }

            try
            {
                MX370xWeb.DefaultResponse response = this.sys.ExtDigitalIOGetNumberOfPorts(0, out portNumber);
                if (response.iReturnValue != 0)
                {
                    error = "ExtDigitalIOGetNumberOfPorts error: " + response.iReturnValue.ToString();
                    return -1;
                }
            }
            catch (Exception ex)
            {
                error = "ExtDigitalIOGetNumberOfPorts exception: " + ex.Message;
                return -2;
            }
            return 0;
        }

        /// <summary>
        /// Get the actual digital input filter configuration for a given port
        /// </summary>
        /// <param name="port"></param>
        /// <param name="filterTimer"></param>
        /// <param name="filterEnabled"></param>
        /// <param name="error"></param>
        /// <returns></returns>
        public int ExtDigitalIOGetInputsFilterConfiguration(ulong port, out ulong filterTime, out ulong filterEnabled, ref string error)
        {
            // initialisation
            filterTime = 0;
            filterEnabled = 0;
            error = string.Empty;

            // Check
            if (this.sys == null)
            {
                error = "System is null. Check the IP address.";
                return -3;
            }

            try
            {
                MX370xWeb.DefaultResponse response = this.sys.ExtDigitalIOGetInputsFilterConfiguration(port, 0, out filterTime, out filterEnabled);
                if (response.iReturnValue != 0)
                {
                    error = "ExtDigitalIOGetInputsFilterConfiguration error: " + response.iReturnValue.ToString();
                    return -1;
                }
            }
            catch (Exception ex)
            {
                error = "ExtDigitalIOGetInputsFilterConfiguration exception: " + ex.Message;
                return -2;
            }
            return 0;
        }

        /// <summary>
        /// Enable or disable the input filter on a given port
        /// </summary>
        /// <param name="port"></param>
        /// <param name="enabled"></param>
        /// <param name="error"></param>
        /// <returns></returns>
        public int ExtDigitalIOEnableDisableInputsFilter(ulong port, ulong enabled, ref string error)
        {
            // initialisation
            error = string.Empty;

            // Check
            if (this.sys == null)
            {
                error = "System is null. Check the IP address.";
                return -3;
            }

            try
            {
                int syserrno = 0;
                int retval = this.sys.ExtDigitalIOEnableDisableInputsFilter(port, enabled, 0, out syserrno);
                if (retval != 0)
                {
                    error = "ExtDigitalIOEnableDisableInputsFilter error: " + retval.ToString();
                    return -1;
                }
            }
            catch (Exception ex)
            {
                error = "ExtDigitalIOEnableDisableInputsFilter exception: " + ex.Message;
                return -2;
            }
            return 0;
        }

        /// <summary>
        /// Set the filter time
        /// </summary>
        /// <param name="filterTime"></param>
        /// <returns></returns>
        public int ExtDigitalIOSetInputsFilterTime(ulong filterTime, ref string error)
        {
            // initialisation
            error = string.Empty;

            // Check
            if (this.sys == null)
            {
                error = "System is null. Check the IP address.";
                return -3;
            }

            try
            {
                int syserrno = 0;
                int retval = this.sys.ExtDigitalIOSetInputsFilterTime(filterTime, 0, out syserrno);
                if (retval != 0)
                {
                    error = "ExtDigitalIOSetInputsFilterTime error: " + retval.ToString();
                    return -1;
                }
            }
            catch (Exception ex)
            {
                error = "ExtDigitalIOSetInputsFilterTime exception: " + ex.Message;
                return -2;
            }
            return 0;
        }
    }
}
