using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace Filter
{
    public partial class MainForm : Form
    {
        /// <summary>
        /// MSX-Exxxx system
        /// </summary>
        private MSXE3701 mx = null;

        /// <summary>
        /// Constructor
        /// </summary>
        public MainForm()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Occurs the first time the form is loaded
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MainForm_Load(object sender, EventArgs e)
        {
            // Load the parameters
            this.ipTextBox.Text = Properties.Settings.Default.ip;

            // Some default
            this.enabledDisabledComboBox.SelectedIndex = 0;
        }

        /// <summary>
        /// Occurs when the form is being closed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            // Save the parameters
            Properties.Settings.Default.ip = this.ipTextBox.Text;
            Properties.Settings.Default.Save();
        }

        /// <summary>
        /// Click on the initialisation button
        /// Since the initialisation may take time, we do it in a background worker
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void initButton_Click(object sender, EventArgs e)
        {
            this.initButton.Enabled = false;
            this.updateTimer.Stop();

            // Create the mx object
            this.mx = new MSXE3701(this.ipTextBox.Text);

            setConfigurationGroupBox.Enabled = true;

            this.updateTimer.Start();
        }

        /// <summary>
        /// Add a log
        /// </summary>
        /// <param name="log"></param>
        private void AddLog(String log)
        {
            this.logTextBox.Text += DateTime.Now.ToString() + "\t" + log + Environment.NewLine;
            this.logTextBox.SelectionStart = this.logTextBox.Text.Length;
            this.logTextBox.ScrollToCaret();
        }

        /// <summary>
        /// Timer ticks
        /// Update the current filter configuration
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void updateTimer_Tick(object sender, EventArgs e)
        {
            if (this.mx == null)
                return;

            int retval = 0;
            string error = string.Empty;
            ulong filterTime = 0;
            ulong filterEnabled = 0;
            

            // Get the actual configuration
            if ((retval = this.mx.ExtDigitalIOGetInputsFilterConfiguration(0, out filterTime, out filterEnabled, ref error)) != 0)
            {
                this.AddLog(error);
                updateTimer.Stop();
                return;
            }

            if (filterEnabled == 0)
            {
                this.currentFilterStateTextBox.Text = "Disabled.";
                this.currentFilterTimeMicroSecTextBox.Text = string.Empty;
                this.currentFilterTimeTextBox.Text = string.Empty;
            }
            else
            {
                this.currentFilterStateTextBox.Text = "Enabled.";
                this.currentFilterTimeTextBox.Text = filterTime.ToString();

                int filterUs = (int)filterTime * 20;
                this.currentFilterTimeMicroSecTextBox.Text = filterUs.ToString();
            }
        }

        /// <summary>
        /// Click on set configuration
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void setConfigurationButton_Click(object sender, EventArgs e)
        {
            ulong enabled = 0;
            int retval = 0;
            string error = string.Empty;
            ulong filterTime = 0;

            // Check
            if ( this.enabledDisabledComboBox.SelectedIndex < 0)
                return;

            try
            {
                filterTime = Convert.ToUInt32(this.filterTimeTextBox.Text);
            }
            catch (Exception) { return; }

            enabled = (ulong)this.enabledDisabledComboBox.SelectedIndex;

            // Enable/Disable fiilter
            if ((retval = this.mx.ExtDigitalIOEnableDisableInputsFilter(0, enabled, ref error)) != 0)
            {
                this.AddLog(error);
                return;
            }

            // Set the filter time
            if ((retval = this.mx.ExtDigitalIOSetInputsFilterTime(filterTime, ref error)) != 0)
            {
                this.AddLog(error);
                return;
            }
        }
    }
}