using System;
using System.Collections.Generic;
using System.Text;

namespace Information
{
    public class MSXE3701
    {
        /// <summary>
        /// Web reference to the system
        /// Enables access on it
        /// </summary>
        private MX370xWeb.MX370x sys;

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="ip">IP address of the system</param>
        public MSXE3701(String ip)
        {
            // Construct the web reference
            this.sys = new MX370xWeb.MX370x();
            try
            {
                this.sys.Url = "http://" + ip + ":5555";
                this.sys.Timeout = 2000;
            }
            catch (Exception)
            {
                this.sys = null;
            }
        }

        /// <summary>
        /// Get the number of external digital I/O available
        /// </summary>
        /// <param name="channelNumber"></param>
        /// <param name="error"></param>
        /// <returns></returns>
        public int ExtDigitalIOGetNumberOfChannels(out ulong channelNumber, ref string error)
        {
            // initialisation
            channelNumber = 0;
            error = string.Empty;

            // Check
            if (this.sys == null)
            {
                error = "System is null. Check the IP address.";
                return -3;
            }

            try
            {
                MX370xWeb.DefaultResponse response = this.sys.ExtDigitalIOGetNumberOfChannels(0, out channelNumber);
                if (response.iReturnValue != 0)
                {
                    error = "ExtDigitalIOGetNumberOfChannels error: " + response.iReturnValue.ToString();
                    return -1;
                }
            }
            catch (Exception ex)
            {
                error = "ExtDigitalIOGetNumberOfChannels exception: " + ex.Message;
                return -2;
            }
            return 0;
        }

        /// <summary>
        /// Get the number of external digital port available
        /// </summary>
        /// <param name="portNumber"></param>
        /// <param name="error"></param>
        /// <returns></returns>
        public int ExtDigitalIOGetNumberOfPorts(out ulong portNumber, ref string error)
        {
            // initialisation
            portNumber = 0;
            error = string.Empty;

            // Check
            if (this.sys == null)
            {
                error = "System is null. Check the IP address.";
                return -3;
            }

            try
            {
                MX370xWeb.DefaultResponse response = this.sys.ExtDigitalIOGetNumberOfPorts(0, out portNumber);
                if (response.iReturnValue != 0)
                {
                    error = "ExtDigitalIOGetNumberOfPorts error: " + response.iReturnValue.ToString();
                    return -1;
                }
            }
            catch (Exception ex)
            {
                error = "ExtDigitalIOGetNumberOfPorts exception: " + ex.Message;
                return -2;
            }
            return 0;
        }

        /// <summary>
        /// Get the number of channels for a given port
        /// </summary>
        /// <param name="port"></param>
        /// <param name="channelNumber"></param>
        /// <param name="error"></param>
        /// <returns></returns>
        public int ExtDigitalIOGetNumberOfChannelsPerPort(ulong port, out ulong channelNumber, ref string error)
        {
            // initialisation
            channelNumber = 0;
            error = string.Empty;

            // Check
            if (this.sys == null)
            {
                error = "System is null. Check the IP address.";
                return -3;
            }

            try
            {
                MX370xWeb.DefaultResponse response = this.sys.ExtDigitalIOGetNumberOfChannelsPerPort(port, 0, out channelNumber);
                if (response.iReturnValue != 0)
                {
                    error = "ExtDigitalIOGetNumberOfChannelsPerPort error: " + response.iReturnValue.ToString();
                    return -1;
                }
            }
            catch (Exception ex)
            {
                error = "ExtDigitalIOGetNumberOfChannelsPerPort exception: " + ex.Message;
                return -2;
            }
            return 0;
        }

        /// <summary>
        /// Get the direction of a given port
        /// </summary>
        /// <param name="port"></param>
        /// <param name="direction"></param>
        /// <param name="error"></param>
        /// <returns></returns>
        public int ExtDigitalIOGetPortDirections(ulong port, out ulong direction, ref string error)
        {
            // initialisation
            direction = 0;
            error = string.Empty;

            // Check
            if (this.sys == null)
            {
                error = "System is null. Check the IP address.";
                return -3;
            }

            try
            {
                MX370xWeb.DefaultResponse response = this.sys.ExtDigitalIOGetPortDirections(port, 0, out direction);
                if (response.iReturnValue != 0)
                {
                    error = "ExtDigitalIOGetPortDirections error: " + response.iReturnValue.ToString();
                    return -1;
                }
            }
            catch (Exception ex)
            {
                error = "ExtDigitalIOGetPortDirections exception: " + ex.Message;
                return -2;
            }
            return 0;
        }
    }
}
